## Title: Function to compute regression to the mean for bivariate Poisson distribution
## Author: Manzoor Khan
## Date: 7 March 2018

### Evaluation of RTM effects for different cut-off points in both tails

RTM.Poisson=function(Cut.off,x1,x2){
  RTM.Right=c();RTM.Left=c();x=c();y=c();xl=c();yl=c();
for(i in 1:length(Cut.off)){
  x=x1[x1>Cut.off[i]];
  y=x2[x1>Cut.off[i]];
  xl=x1[x1<Cut.off[i]];
  yl=x2[x1<Cut.off[i]];
  RTM.Right[i]=mean(x)-mean(y);
  RTM.Left[i]=mean(yl)-mean(xl);
}
  RTM=round(cbind(Cut.off,RTM.Right,RTM.Left),3)
  return(RTM);
}
##################################### Graph of RTM

### Grap of RTM for different left and right cut-off points
RTM.plot=function(Cut.off,x1,x2){
  RTM.Right=c();RTM.Left=c();x=c();y=c();xl=c();yl=c();
  for(i in 1:length(Cut.off)){
    x=x1[x1>Cut.off[i]];
    y=x2[x1>Cut.off[i]];
    xl=x1[x1<Cut.off[i]];
    yl=x2[x1<Cut.off[i]];
    RTM.Right[i]=mean(x)-mean(y);
    RTM.Left[i]=mean(yl)-mean(xl);
  }
plot(Cut.off,RTM.Right,type="l",lty=1,lwd=2,col="red",ylab=expression(paste(R(y['0'],y))),xlab=expression(paste(y['0'])),cex.lab=1, cex.axis=1, cex.main=2, cex.sub=1.5)
lines(y0,RTM.Left,type="l",lty=1,lwd=2,col="green")
legend("topleft", inset=.05,cex = 1,title="",c("right cut-off point", "left cut-off point"),horiz=FALSE,
       lty=c(1,1),lwd=c(2,2),col=c("red","green"),bg="grey96",bty="n")
}
##########

### Function to evaluate TBVP prob mass function at sample point (y1,y2)
TBVP=function(th1,th2,th0,Tr,z1,z2){
  j=0:min(z1,z2);
  ADD=c();
  for(i in 1:length(j)){
    num=exp(-th0-th1-th2)*(th1^(z1-j[i]))*(th2^(z2-j[i]))*(th0^j[i]);
    den=factorial(z1-j[i])*factorial(z2-j[i])*factorial(j[i])*(1-ppois(Tr,th0+th1));
    ADD[i]=num/den;}
  return(sum(ADD));}
#######################################

### MLE of Truncated Univariate Poisson  distribution i.e. thet0+theta1 to estimate
### parameters of a Truncated BVP

### Truncated Univariate Poisson probability mass function
UP=function(Tr,Y,lmdaa){
  TrPois=dpois(Y,lmdaa)/(1-ppois(Tr,lmdaa));
  return(TrPois)}
###########################################

### Function to find MLE of the univariate Poisson distribution, i.e., theta0+theta1
t0pt1=function(Tr,X1){
  y11=X1[which(X1>Tr)];llf=c();n=length(y11);
  m=mean(y11);It0pt1=seq(0.01,m,0.01);limda=c();
  for(i in 1:length(It0pt1)){
    limda[i]=mean(y11)*(1-ppois(Tr,It0pt1[i]))/(1-ppois(Tr-1,It0pt1[i]));
    lf1=c();
    for(ii in 1:n){
      lf1[ii]=log(UP(Tr,y11[ii],limda[i]));}
    llf[i]=sum(lf1);}
  plot(It0pt1,llf,"l",main="plot of log-likelihood function");
  Trt0pt1=It0pt1[which(llf==max(llf))];
  return(Trt0pt1)}
#########################################

### Log likelihood function of the truncated bivariate Poisson distribution
### for finding MLEs of the TBVP 
Treatment.RTM=function(Tr,X1,X2){
  y11=X1[which(X1>Tr)];
  y22=X2[which(X1>Tr)];
  LLF=c();R=(1-ppois(Tr-1,t0pt1(Tr,X1)))/(1-ppois(Tr,t0pt1(Tr,X1)));
  n=length(y22);m=min(mean(y22)/R,mean(y11)/R);t0=seq(0.01,m,0.01);
  t1=c();t2=c();
for(i in 1:length(t0)){
  t1[i]=mean(y11)/R-t0[i];
  t2[i]=mean(y22)-t0[i]*R;
  lf=c();
  for(ii in 1:n){
    lf[ii]=log(TBVP(t1[i],t2[i],t0[i],Tr,y11[ii],y22[ii]));}
  LLF[i]=sum(lf);}

  cat("========================================\n")
cat("Estimation of the parameters of the \n bivariate Poisson distribution\n") 
cat("========================================\n")
plot(t0,LLF,"l",main="plot of log-likelihood function");
theta0=t0[which(LLF==max(LLF))];
cat(paste("Estimate of theta0=",theta0))
cat("\n")
cat("========================================\n")
theta1=mean(y11)/R-theta0;
cat(paste("Estimate of theta1=",round(theta1,3)));
cat("\n")
cat("========================================\n")
theta2=mean(y22)-theta0*R;
cat(paste("Estimate of theta2=",round(theta2,3)));
cat("\n")
cat("========================================\n")
cat("Estimation of total, treatment and RTM\n Effects\n")
cat("========================================\n")
TotEff=R*theta1-theta1+(theta1-theta2);
cat(paste("Total Effect=",round(TotEff,3)));
cat("\n")
cat("========================================\n")
TreatmentEffect=theta1-theta2;
cat(paste("Treatment Effect=",round(TreatmentEffect,3)))
cat("\n")
cat("========================================\n")
RTMEffect=R*theta1-theta1;
cat(paste("RTM Effect=",round(RTMEffect,3)))
}
